import json
import os
import shutil
import uuid

from launcher import config
from launcher.config.conf import LaunchPart
from launcher.constants import instances_folder_name
from launcher.webview.events import custom_event, LogType


def add_instance():
    instances_path = os.path.join(os.getcwd(), instances_folder_name)
    if not os.path.exists(instances_path):
        os.makedirs(instances_path)
    instance_folder_name = str(uuid.uuid4())
    instance_folder_path = os.path.join(instances_path, instance_folder_name)
    if os.path.exists(instance_folder_path):
        raise Exception("创建实例目录失败")
    os.makedirs(instance_folder_path)
    custom_event(LogType.info, f"创建实例目录成功: {instance_folder_path}")
    return instance_folder_path


def migrate_instance(source_folder):
    """通用配置迁移方法
    Args:
        source_folder: 源配置目录路径
        target_folder: 目标配置目录路径
    """
    try:
        if not os.path.exists(source_folder):
            custom_event(LogType.error, f"源配置目录不存在: {source_folder}")
            return {"status": False, "message": f"源配置目录不存在{source_folder}"}

        target_folder = add_instance()

        # 需要复制的目录和文件列表
        copy_items = [("tmp", True), ("conf.yml", False), ("plan.json", False)]

        for item, is_dir in copy_items:
            src = os.path.join(source_folder, item)
            dst = os.path.join(target_folder, item)

            if is_dir:
                if os.path.exists(src):
                    shutil.copytree(src, dst, dirs_exist_ok=True)
            else:
                if os.path.exists(src):
                    shutil.copy2(src, dst)

        custom_event(LogType.info, f"{source_folder} 配置成功迁移至 {target_folder}")
        return {"status": True, "data": target_folder, "message": "配置迁移成功"}

    except Exception as e:
        custom_event(LogType.error, f"配置迁移失败: {str(e)}")
        return {"status": False, "message": str(e)}


def migrate_instances_config():
    try:
        config_path = os.path.join(os.getcwd(), "mower-ng", "instances.json")

        if not os.path.exists(config_path):
            custom_event(LogType.error, f"多开配置文件不存在{config_path}")
            return {"status": False, "message": "多开配置文件不存在"}

        with open(config_path, "r", encoding="utf-8") as f:
            instances_data = json.loads(f.read())
            custom_event(LogType.info, f"读取多开配置: {instances_data}")

        valid_instances = []
        error_messages = []

        for index, item in enumerate(instances_data, 1):
            try:
                # 基础结构验证
                if not isinstance(item, dict):
                    raise ValueError("配置项必须是字典类型")

                # 必填字段检查
                required_fields = ["name", "path"]
                for field in required_fields:
                    if field not in item:
                        raise ValueError(f"缺少必要字段: {field}")

                # 路径有效性验证
                if not os.path.exists(item["path"]):
                    raise ValueError(f"配置路径不存在{item['path']}")

                # 执行配置迁移
                migration_result = migrate_instance(item["path"])

                if not migration_result["status"]:
                    raise Exception(f"路径迁移失败: {migration_result['message']}")

                # 转换为实例模型
                instance = LaunchPart.Instance(
                    name=item["name"].strip(), path=migration_result["data"]
                )

                valid_instances.append(instance)

            except Exception as e:
                error_msg = f"第{index}项: {str(e)}"
                error_messages.append(error_msg)
                custom_event(LogType.error, error_msg)

        # 保存有效配置
        if valid_instances:
            config.conf.instances.extend(valid_instances)
            config.save_conf()

        message = f"成功导入{len(valid_instances)}个实例" + (
            f"，存在{len(error_messages)}个错误项" if error_messages else ""
        )
        custom_event(LogType.info, message)
        return {
            "status": not bool(error_messages),
            "data": len(valid_instances),
            "message": message,
        }

    except json.JSONDecodeError as e:
        custom_event(LogType.error, f"JSON解析失败: {str(e)}")
        return {"status": False, "message": "配置文件格式错误"}
    except Exception as e:
        custom_event(LogType.error, f"迁移多开配置失败: {str(e)}")
        return {"status": False, "message": str(e)}
